package fr.jeremylebrun.ld41.core;

import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;

public abstract class Shared {
	
	public static  boolean DEBUG = false;
	public static ShapeRenderer SHAPE_RENDERER;
	
	/*
	 * CONSTANTES DE DIRECTION
	 */
	public static final int DIR_DOWN = 0;
	public static final int DIR_LEFT = 1;
	public static final int DIR_RIGHT = 2;
	public static final int DIR_UP = 3;
	public static final int DIR_UP_LEFT = 4;
	public static final int DIR_UP_RIGHT = 5;
	public static final int DIR_DOWN_LEFT = 6;
	public static final int DIR_DOWN_RIGHT = 7;
	
	/*
	 * CONSTANTES DE JEU
	 */
	public static final int GAME_WIDTH = 640;
	public static final int GAME_HEIGHT = 360;
	public static final String GAME_TITLE = "LD41";
	
	/*
	 * OBJETS PARTAGES
	 */
	public static SpriteBatch GAME_BATCH;
	public static SpriteBatch MENU_BATCH;
	
	// FONTS
	public static BitmapFont FONT, FONT_SMALL, FONT_BIG;
	
	public static Animation<TextureRegion>[] BLOOD_ANIMATIONS;
	

	// POINTS
	
	/* ***********************************************
	 * TEXTURES
	 */
	
	// HUD 
	
	
	
	// FONCTIONS UTILES
	/**
	 * @param textureRegion la grille de sprites
	 * @param nbFrames le nombre de frame
	 * @param line l'index de la ligne
	 * @param col l'index de la colonne
	 * @param frameDuration la dure de chaque frame
	 * @return l'objet animation cr
	 */
	public static Animation<TextureRegion> loadAnimation(TextureRegion[][] textureRegion, int nbFrames, int line, float frameDuration, PlayMode mode){
		TextureRegion[] frames = new TextureRegion[nbFrames];
		for (int i = 0; i < nbFrames; i++) {
			frames[i] = textureRegion[line][i];
		}
		Animation<TextureRegion> a = new Animation<>(frameDuration, frames);
		a.setPlayMode(mode);
		return a;
	}
	
	
	/**
	 * @param dir1
	 * @param dir2
	 * @return vrai si les 2 directions sont opposes
	 */
	public static boolean isOpositeDirection(int dir1, int dir2){
		if((dir1 == Shared.DIR_DOWN && dir2 == Shared.DIR_UP) || (dir2 == Shared.DIR_DOWN && dir1 == Shared.DIR_UP)){
			return true;
		}
		if((dir1 == Shared.DIR_LEFT && dir2 == Shared.DIR_RIGHT) || (dir2 == Shared.DIR_RIGHT && dir1 == Shared.DIR_LEFT)){
			return true;
		}
		return false;
	}
	
	/**
	 * 
	 * @param number
	 * @param min
	 * @param max
	 * @return le parametre dans les limites.(si le parametre est superieur au max ou inferieur au min, on retourne le max ou le min.)
	 */
	public static float stayInLimit(float number, float min, float max){
		float f = number;
		if(f>max){
			f = max;
		}else if(f<min){
			f = min;
		}
		return f;
	}
	
	public static int getDirectionFromDeplacement(float dx, float dy) {
		int direction = Shared.DIR_UP;
		if(dy == 0) {
			if(dx > 0) {
				direction = Shared.DIR_RIGHT;
			}else if(dx < 0) {
				direction = Shared.DIR_LEFT;
			}
		}else if(dx == 0) {
			if(dy > 0) {
				direction = Shared.DIR_UP;
			}else if(dy < 0) {
				direction = Shared.DIR_DOWN;
			}
		}else {
			if(dx > 0 && dy > 0) {
				direction = Shared.DIR_UP_RIGHT;
			}else if(dx < 0  && dy > 0) {
				direction = Shared.DIR_UP_LEFT;
			}else if(dx < 0  && dy < 0) {
				direction = Shared.DIR_DOWN_LEFT;
			}else if(dx > 0  && dy < 0) {
				direction = Shared.DIR_DOWN_RIGHT;
			}
		}
		return direction;
	}
	

	public static float getRotationFromDirection( int direction) {
		float rotation = 0;
		if(direction == Shared.DIR_UP) {
			rotation = 0;
		}else if(direction == Shared.DIR_UP_RIGHT) {
			rotation = -45;
		}else if(direction == Shared.DIR_UP_LEFT) {
			rotation = 45;
		}else if(direction == Shared.DIR_RIGHT) {
			rotation = -90;
		}else if(direction == Shared.DIR_DOWN) {
			rotation = 180;
		}else if(direction == Shared.DIR_DOWN_LEFT) {
			rotation = 135;
		}else if(direction == Shared.DIR_DOWN_RIGHT) {
			rotation = -135;
		}else if(direction == Shared.DIR_LEFT) {
			rotation = 90;
		}else {
			rotation = 0;
		}
		return rotation;
	}
	

	public static void initBloodAnimation() {
		if(BLOOD_ANIMATIONS == null){
			TextureRegion[][] bloodTR = TextureRegion.split(Ressources.TEXTURE_ANIM_BLOOD, 16, 16);
			BLOOD_ANIMATIONS = new Animation[8];
			for(int i=0;i<8;i++){
				BLOOD_ANIMATIONS[i] = Shared.loadAnimation(bloodTR, 6, i, 0.08f,PlayMode.NORMAL);
			}
		}
	}
}
