package fr.jeremylebrun.ld41.entities;


import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;

import fr.jeremylebrun.ld41.core.Ressources;
import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.menus.GameMenu;

public class Player extends Entity {

	private Animation<TextureRegion>[] runAnimation  = new Animation[2];
	private Animation<TextureRegion>[] idleAnimation   = new Animation[2];
	private Animation<TextureRegion>[] fallAnimation  = new Animation[2];
	
	private float statetime = 0f;
	private float rotation = 0;
	private int direction= Shared.DIR_UP;
	private float currentSpeed = 1f;
	private float maxSpeed = 3f;
	
	private State state;
	private Vector2 lastMove;
	
	private float jumpForce = 1;
	private float jumpDx = 0;
	private float jumpDy =0;
	private float jumpHeight=0;
	private float dieTimer = 0f;
	private boolean jumpAscending = true;
	
	private int gunLevel = 1;
	private int coldownHit = 0;
	
	private Vector2 lastWantDir;
	
	private int life = 3;
	private int maxLife = 3;
	
	private float flashRed = 0f;
	
	
	public Player(float x, float y) {
		super(x, y, 64, 64,0);
		lastMove = new Vector2(0,0);
		lastWantDir = new Vector2(0,0);
		gunLevel = 0;
	}

	
	/**
	 * Initialise l'entite
	 * @param game
	 */
	public void init(GameMenu game) {
		TextureRegion[][] textureRegion = TextureRegion.split(Ressources.TX_PLAYER,64,64);
		
		for(int i=0;i<2;i++) {
			TextureRegion[] runFrames = new TextureRegion[4];
			TextureRegion[] idleFrames = new TextureRegion[1];
			TextureRegion[] fallFrames = new TextureRegion[1];
			for (int j = 0; j < 4; j++) {
				runFrames[j] = textureRegion[i][j];
			}
			idleFrames[0] = textureRegion[i][0];
			fallFrames[0] = textureRegion[i][4];
			
			runAnimation[i] = new Animation<TextureRegion>(0.15f, runFrames);
			runAnimation[i].setPlayMode(Animation.PlayMode.LOOP);
			
			idleAnimation[i] = new Animation<TextureRegion>(0.15f, idleFrames);
			idleAnimation[i].setPlayMode(Animation.PlayMode.LOOP);
			
			fallAnimation[i] = new Animation<TextureRegion>(0.15f, fallFrames);
			fallAnimation[i].setPlayMode(Animation.PlayMode.LOOP);
		}
		
		state = State.IDLE;
		
		lstCollisions.add(new Vector2(x-10, y+10));
		lstCollisions.add(new Vector2(x+10, y+10));
		lstCollisions.add(new Vector2(x-10, y-10));
		lstCollisions.add(new Vector2(x+10, y-10));
		
		currentSpeed = 1;
		super.init(game);
	}

	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void render(GameMenu game) {
		TextureRegion currentFrame;
		if(state == State.IDLE) {
			currentFrame = idleAnimation[gunLevel].getKeyFrame(statetime);
		}else if(state == State.MOVE) {
			currentFrame = runAnimation[gunLevel].getKeyFrame(statetime);
		}else if(state == State.FALL || state == State.FALL_DIE) {
			currentFrame = fallAnimation[gunLevel].getKeyFrame(statetime);
		}else {
			currentFrame = idleAnimation[gunLevel].getKeyFrame(statetime);
		}
		
		if(state == State.FALL_DIE) {
			Shared.GAME_BATCH.setColor(dieTimer, dieTimer, dieTimer, dieTimer);
		}else if(flashRed>0f) {
			Shared.GAME_BATCH.setColor(1f, 1f-flashRed, 1f-flashRed, 1f);
			flashRed-=0.05f;
		}
		Shared.GAME_BATCH.draw(currentFrame, (int)(x-width/2),  (int)(y-height/2),32,32,width,height,1,1,rotation);
		super.render(game);
		
		Shared.GAME_BATCH.setColor(1f, 1f, 1f, 1f);
		
		statetime += Gdx.graphics.getDeltaTime(); 

	}

	/**
	 * Met a jour l'entite
	 * @param game
	 */
	public void update(GameMenu game) {
		super.update(game);
		
		// GESTION DE LA MORT 
		if(dieTimer > 0f) {
			dieTimer -= 0.01f;
			if(dieTimer <=0f) {
				dieTimer = 0f;
				life --;
				if(life <=0) {
					game.getHud().gameOver();
				}else {
					game.goToCheckpoint();
				}
			
			}
			// ON ARRETE L'UPDATE ICI
			return;
		}
		
		if(coldownHit>0) {
			coldownHit--;
		}
		
		if(state == State.IDLE) {
			lastMove.x = 0f;
			lastMove.y = 0f;
			lastWantDir.x = 0f;
			lastWantDir.y = 0f;
		}
		
		if(state != State.FALL && game.getMap().onGround(this)) {
			if(game.camera.zoom<1f) {
				game.camera.zoom+=0.005f;
			}
			if(state == State.FALL) {
				state = State.IDLE;
			}
			this.updateDeplacementInput(game);
			this.updateJumpInput(game);
		}else {
			this.updateAir(game);
		}
		
		this.updateShoot(game);
		
		this.updateRotation();
	}
	
	private void updateRotation() {
		if(direction == Shared.DIR_UP) {
			rotation = 0;
		}else if(direction == Shared.DIR_UP_RIGHT) {
			rotation = -45;
		}else if(direction == Shared.DIR_UP_LEFT) {
			rotation = 45;
		}else if(direction == Shared.DIR_RIGHT) {
			rotation = -90;
		}else if(direction == Shared.DIR_DOWN) {
			rotation = 180;
		}else if(direction == Shared.DIR_DOWN_LEFT) {
			rotation = 135;
		}else if(direction == Shared.DIR_DOWN_RIGHT) {
			rotation = -135;
		}else if(direction == Shared.DIR_LEFT) {
			rotation = 90;
		}else {
			rotation = 0;
		}
	}
	
	private void updateDeplacementInput(GameMenu game) {
		boolean directionLock = Gdx.input.isKeyPressed(Input.Keys.CONTROL_LEFT);
		float wantDirX = 0;
		float wantDirY = 0;
		
		
		// Mise a jour du deplacement
		if(Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			wantDirX += -1;
		}
		if(Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			wantDirX += 1;
		}
		if(Gdx.input.isKeyPressed(Input.Keys.UP)) {
			wantDirY += 1;
		}
		if(Gdx.input.isKeyPressed(Input.Keys.DOWN)) {
			wantDirY += -1;
		}
		wantDirX *=currentSpeed;
		wantDirY*=currentSpeed;
//		System.out.println(wantDirX +" | "+wantDirY+" | "+currentSpeed);
		
		// Mise a jour de la direction
		if(!directionLock) {
			if(wantDirY == 0) {
				if(wantDirX > 0) {
					direction = Shared.DIR_RIGHT;
				}else if(wantDirX < 0) {
					direction = Shared.DIR_LEFT;
				}
			}else if(wantDirX == 0) {
				if(wantDirY > 0) {
					direction = Shared.DIR_UP;
				}else if(wantDirY < 0) {
					direction = Shared.DIR_DOWN;
				}
			}else {
				if(wantDirX > 0 && wantDirY > 0) {
					direction = Shared.DIR_UP_RIGHT;
				}else if(wantDirX < 0  && wantDirY > 0) {
					direction = Shared.DIR_UP_LEFT;
				}else if(wantDirX < 0  && wantDirY < 0) {
					direction = Shared.DIR_DOWN_LEFT;
				}else if(wantDirX > 0  && wantDirY < 0) {
					direction = Shared.DIR_DOWN_RIGHT;
				}
			}
		}
		
		if(wantDirX != 0 && wantDirY != 0) {
			// On se deplace en diagonal : on diminue la vitesse;
			wantDirX *= 0.7;
			wantDirY *= 0.7;
		}
		
		lastWantDir.x = wantDirX;
		lastWantDir.y = wantDirY;
		
		// Mise a jour de l'etat et deplacement
		if(wantDirX != 0 || wantDirY != 0) {
			if(state == State.IDLE) {
				state = State.MOVE;
				currentSpeed = 1;
			}else if(currentSpeed<maxSpeed){
				if(currentSpeed<2) {
					currentSpeed += 0.1f;
				}else if(currentSpeed<2.5) {
					currentSpeed += 0.05f;
				}else {
					currentSpeed += 0.01f;
				}
				
			}
			
			
			
			// Deplacement
			if(!move(game, wantDirX, wantDirY, false ,true)) {
				currentSpeed = 1;
				lastMove.x=0;
				lastMove.y=0;
			}else {
				if(!game.getMap().onGround(this)) {
					state = State.FALL;
					jumpAscending = false;
					jumpDx = 0f;
					jumpDy = 0f;
					jumpForce = 0f;
					jumpHeight = 12f;
				}
			}
			
		}else if(state == State.MOVE) {
			state = State.IDLE;
			currentSpeed = 1;
		}
	}
	
	private void updateJumpInput(GameMenu game) {
		if(state != State.FALL) {
			// au sol
			if(Gdx.input.isKeyJustPressed(Input.Keys.SPACE)) {
				Ressources.SOUND_JUMP.play(0.5f);
				game.getTmpLayerEntities(level).add(new GameAnimation(GameAnimationType.PLAYER_JUMP, x, y, 64, 64, level,rotation));
				
				// init du saut
				jumpForce = ((currentSpeed-1f)/3)+1.5f;
				
				jumpDx = lastWantDir.x==0?0:Math.signum(lastWantDir.x) * Math.max(jumpForce, 1.7f);
				jumpDy = lastWantDir.y==0?0:Math.signum(lastWantDir.y) * Math.max(jumpForce, 1.7f);
				
				jumpHeight = 0;
				jumpAscending = true;
				
				state = State.FALL;
				
				
//				System.out.println("JUMP "+jumpDx+" | "+jumpDy);
			}
		}
	}
	private void updateAir(GameMenu game) {
		if(state == State.FALL) {
			// deja en l'air
			if(jumpAscending) {
				if(game.camera.zoom>0.9f) {
					game.camera.zoom-=0.005f;
				}
				// phase montante
				if(jumpHeight<20) {
					jumpHeight++;
					if(jumpHeight==12) {
						this.level++;
						if(!game.getMap().onWall(this)) {
							lastMove.x = lastMove.x*0.7f;
							lastMove.y = lastMove.y*0.7f;
						}
						
					}else if(jumpHeight == 20) {
						jumpAscending = false;
						if(!game.getMap().onWall(this)) {
							lastMove.x = lastMove.x*0.7f;
							lastMove.y = lastMove.y*0.7f;
						}
						
					}
				}
				if(move(game,jumpDx,jumpDy,true,true)) {
					jumpDx = lastMove.x;
					jumpDy = lastMove.y;
				}
				
			}else {
				if(game.camera.zoom<1f) {
					game.camera.zoom+=0.005f;
				}
				jumpHeight--;
				if(jumpHeight%12 == 0) {
					if(!game.getMap().onGround(this) ) {
						if(!game.getMap().onWall(this)) {
							lastMove.x = lastMove.x*0.7f;
							lastMove.y = lastMove.y*0.7f;
							if(level>0) {
								level--;
							} else {
								state = State.FALL_DIE;
								Ressources.SOUND_JUMP_FAIL.play();
								dieTimer = 1f;
							}
						}else {
//							System.out.println("ON WALL");
						}
					}else { 
						Ressources.SOUND_JUMP_LAND.play();
						game.getTmpLayerEntities(level).add(new GameAnimation(GameAnimationType.PLAYER_JUMP_LAND, x, y, 64, 64, level,rotation));
						state = State.IDLE;
						jumpDx=0;
						jumpDy = 0;
					}
				}else if(jumpHeight<0) {
					jumpHeight = 20;
				}
				// phase descendante
				if(state == State.FALL && move(game,jumpDx,jumpDy,true,true)) {
					jumpDx = lastMove.x;
					jumpDy = lastMove.y;
				}
			}
		}
	}
	
	private void updateShoot(GameMenu game) {
		if(gunLevel>0 && (Gdx.input.isKeyJustPressed(Keys.X) ||  Gdx.input.isKeyJustPressed(Keys.C)||  Gdx.input.isKeyJustPressed(Keys.V))) {
			Ressources.SOUND_SHOOT.play();
			game.getTmpLayerEntities(level).add(new GameAnimation(GameAnimationType.GUN1_SHOOT, x, y, 64, 64, level,rotation));

			
			float dx = 0;
			float dy = 0;
			float bulletSpeed = 5;
			if(direction == Shared.DIR_LEFT || direction == Shared.DIR_UP_LEFT || direction == Shared.DIR_DOWN_LEFT ) {
				dx = -1;
			}else if(direction == Shared.DIR_RIGHT || direction == Shared.DIR_UP_RIGHT || direction == Shared.DIR_DOWN_RIGHT) {
				dx = 1;
			}
			
			if(direction == Shared.DIR_UP || direction == Shared.DIR_UP_LEFT || direction == Shared.DIR_UP_RIGHT ) {
				dy = 1;
			}else if(direction == Shared.DIR_DOWN || direction == Shared.DIR_DOWN_LEFT || direction == Shared.DIR_DOWN_RIGHT) {
				dy = -1;
			}
			dx *= bulletSpeed;
			dy *= bulletSpeed;
			game.getLayerEntities(level).add(new PlayerBullet(x+(5f*dx), y+(5f*dy),level, dx, dy, gunLevel,rotation));
		}
	}
	
	@Override
	protected void absoluteMove(GameMenu game, float dx, float dy) {
		if(dy != 0) {
			game.translateCamera(0,dy);
		}
		
		lastMove.x = dx;
		lastMove.y = dy;
		super.absoluteMove(game, dx, dy);
	}
	
	public boolean onContact(Entity e) {
		if(e.getLevel()!=level) {
			return false;
		}
		
		if(Math.abs(e.getX()-this.getX()) < 12 &&  Math.abs(e.getY()-this.getY()) < 12) {
			return true;
		}
		return false;
	}
	
	public boolean toFarFromPlayer(Entity e) {
		if(Math.abs(e.getX()-this.getX()) < (Shared.GAME_WIDTH) &&  Math.abs(e.getY()-this.getY()) < (Shared.GAME_HEIGHT)) {
			return false;
		}
		return true;
	}
	public boolean toFarFromPlayerClose(Entity e) {
		if(Math.abs(e.getX()-this.getX()) < ((Shared.GAME_WIDTH/2)) &&  Math.abs(e.getY()-this.getY()) < (((Shared.GAME_HEIGHT/2)+100))) {
			return false;
		}
		return true;
	}
	
	public void hitByEnemy(GameMenu game, Entity e) {
		if(coldownHit<=0 && state != State.FALL) {
			Ressources.SOUND_HURT.play();
			coldownHit = 60; // invulnerabilite de 60 frames
			flashRed = 1f;
			this.life --;
			if(life <=0) {
				game.getHud().gameOver();
			}
		}
	}

	public void setGunLevel(int gunLevel) {
		this.gunLevel = gunLevel;
	}


	public int getLife() {
		return life;
	}


	public int getMaxLife() {
		return maxLife;
	}


	public void goToPosition(float nx, float ny) {
		this.x = nx;
		this.y = ny;
		state = State.IDLE;
		
		lstCollisions.clear();
		lstCollisions.add(new Vector2(x-10, y+10));
		lstCollisions.add(new Vector2(x+10, y+10));
		lstCollisions.add(new Vector2(x-10, y-10));
		lstCollisions.add(new Vector2(x+10, y-10));
		
		currentSpeed = 1;
	}
	
	public void heal(GameMenu game) {
		if(life<maxLife) {
			life++;
			game.getHud().heal();
		}
	}
	
	public void addLife(GameMenu game) {
		maxLife++;
		life++;
		game.getHud().addLife();
	}
	
}
