package fr.jeremylebrun.ld41.menus;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;

import fr.jeremylebrun.ld41.core.Game;
import fr.jeremylebrun.ld41.core.Ressources;
import fr.jeremylebrun.ld41.core.Shared;


public class EndgameMenu implements Menu{

	private float statetime = 0f;

	private GameHud hud;

	private int killScore=0;
	private int bonusScore=0;
	private int dollarsScore=0;
	private int timeScore=0;
	private int totalScore=0;

	private int nbKillIndex=0;
	private int nbBonusIndex=0;
	private int nbDollarsIndex=0;
	private int nbTimeIndex=0;

	private int cptWait = 0;
	private boolean loadComplete = false;

	public EndgameMenu(){
		
	}


	public void init(Game g) {
		killScore=0;
		bonusScore=0;
		dollarsScore=0;
		timeScore=0;
		totalScore=0;
		nbKillIndex=0;
		nbBonusIndex=0;
		nbDollarsIndex=0;
		nbTimeIndex=0;
		
		cptWait = 20;
		loadComplete = false;


	}

	public void update(Game game){
		cptWait--;
		if(cptWait<=0) {
			cptWait = loadComplete?20:10;
			if(!loadComplete) {
				// Ajout des kills
				if(nbKillIndex<hud.getNbKills()) {
					cptWait = 5;
					nbKillIndex++;
					killScore+=25;
					totalScore+=25;
					if(nbKillIndex == hud.getNbKills() ) {
						cptWait = 30;
					}
				}else if(nbDollarsIndex<hud.getNbDollars()) {
					nbDollarsIndex++;
					dollarsScore+=50;
					totalScore+=50;
					if(nbDollarsIndex == hud.getNbDollars() ) {
						cptWait = 30;
					}
				}else if(nbBonusIndex<5) {
					nbBonusIndex++;
					if(nbBonusIndex == 1) {
						bonusScore += hud.isLetter_B()?250:0;
						totalScore += hud.isLetter_B()?250:0;
						cptWait = 30;
					}else if(nbBonusIndex == 2) {
						bonusScore += hud.isLetter_i()?250:0;
						totalScore += hud.isLetter_i()?250:0;
						cptWait = 30;
					}else if(nbBonusIndex == 3) {
						bonusScore += hud.isLetter_g()?250:0;
						totalScore += hud.isLetter_g()?250:0;
						cptWait = 30;
					}else if(nbBonusIndex == 4) {
						bonusScore += hud.isLetter_b2()?250:0;
						totalScore += hud.isLetter_b2()?250:0;
						cptWait = 30;
					}else if(nbBonusIndex == 5 ) {
						bonusScore += hud.isLetter_o()?250:0;
						totalScore += hud.isLetter_o()?250:0;
						cptWait = 50;
					}
				}else if(nbTimeIndex<1) {
					timeScore = Math.max(2000-(3*hud.getTimer()),0);
					totalScore += timeScore;
					nbTimeIndex++;
					cptWait = 50;
				}else {
					loadComplete = true;
					cptWait = 100;
				}
			}
		}
		
		if(loadComplete && Gdx.input.isKeyJustPressed(Keys.SPACE) || Gdx.input.isKeyJustPressed(Keys.ENTER)) {
			Ressources.SOUND_SELEC.play();
			game.goToTitleMenu();
		}

	}

	public void render(Game game){
		statetime += Gdx.graphics.getDeltaTime(); 
		Shared.MENU_BATCH.begin();
		Shared.MENU_BATCH.draw(Ressources.TX_EG_BACKGROUND, 0,0);

		for(int i=0;i<nbKillIndex;i++) {
			Shared.MENU_BATCH.draw(Ressources.TX_EG_KILL, 32+((i%30)*18), 172-((i/30)*18));
		}
		for(int i=0;i<nbDollarsIndex;i++) {
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_DOLLARS, 36+((i%20)*26), 114-((i/20)*18));
		}

		if(nbBonusIndex>=1) {
			if(!hud.isLetter_B()) {
				Shared.MENU_BATCH.setColor(0.2f, 0.2f, 0.2f, 1f);
			}
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_LETTER_B, 28, 55);
			Shared.MENU_BATCH.setColor(1f,1f,1f, 1f);
		}
		if(nbBonusIndex>=2) {
			if(!hud.isLetter_i()) {
				Shared.MENU_BATCH.setColor(0.2f, 0.2f, 0.2f, 1f);
			}
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_LETTER_i, 41, 55);
			Shared.MENU_BATCH.setColor(1f,1f,1f, 1f);
		}
		if(nbBonusIndex>=3) {
			if(!hud.isLetter_g()) {
				Shared.MENU_BATCH.setColor(0.2f, 0.2f, 0.2f, 1f);
			}
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_LETTER_g, 54, 55);
			Shared.MENU_BATCH.setColor(1f,1f,1f, 1f);
		}
		if(nbBonusIndex>=4) {
			if(!hud.isLetter_b2()) {
				Shared.MENU_BATCH.setColor(0.2f, 0.2f, 0.2f, 1f);
			}
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_LETTER_b2, 69, 55);
			Shared.MENU_BATCH.setColor(1f,1f,1f, 1f);
		}
		if(nbBonusIndex>=5) {
			if(!hud.isLetter_o()) {
				Shared.MENU_BATCH.setColor(0.2f, 0.2f, 0.2f, 1f);
			}
			Shared.MENU_BATCH.draw(Ressources.TX_ITEM_LETTER_o, 83, 55);
			Shared.MENU_BATCH.setColor(1f,1f,1f, 1f);
		}
		Shared.FONT.draw(Shared.MENU_BATCH, getScoreString(killScore), 128,212);
		Shared.FONT.draw(Shared.MENU_BATCH, getScoreString(dollarsScore), 128,156);
		Shared.FONT.draw(Shared.MENU_BATCH, getScoreString(bonusScore), 128,98);
		Shared.FONT.draw(Shared.MENU_BATCH, getScoreString(timeScore), 128,40);

		if(loadComplete) {
			Shared.FONT_BIG.setColor(1f,0.9f,0.3f,1f);
		}else {
			Shared.FONT_BIG.setColor(0.5f,0.5f,0.5f,1f);
		}
		Shared.FONT_BIG.draw(Shared.MENU_BATCH, getScoreString(totalScore), 524,40);
		Shared.FONT_BIG.setColor(1f,1f,1f, 1f);

		if(loadComplete) {
			Shared.FONT.draw(Shared.MENU_BATCH, "Press 'SPACE' to exit" , 200,340+(cptWait>=10?0:1));
		}

		Shared.MENU_BATCH.end();
	}

	public void setHud(GameHud hud) {
		this.hud = hud;
	}

	private String getScoreString(int score) {
		String s = ""+score;
		while(s.length()<7) {
			s = "0"+s;
		}
		return s;
	}

}
