package fr.jeremylebrun.ld41.menus;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.math.Vector2;

import fr.jeremylebrun.ld41.core.Game;
import fr.jeremylebrun.ld41.core.Ressources;
import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.entities.Entity;
import fr.jeremylebrun.ld41.entities.Player;
import fr.jeremylebrun.ld41.entities.Zombi;
import fr.jeremylebrun.ld41.map.GameMap;
import fr.jeremylebrun.ld41.map.GameMapItem;
import fr.jeremylebrun.ld41.map.MapItem;
import fr.jeremylebrun.ld41.map.ParralaxMap;


public class GameMenu implements Menu {

	private GameMap level1;
	public OrthographicCamera camera;

	private List<Entity> layer0_entities;
	private List<Entity> layer1_entities;
	private List<Entity> layer2_entities;
	private List<Entity> tmp_layer0_entities;
	private List<Entity> tmp_layer1_entities;
	private List<Entity> tmp_layer2_entities;

	private List<Vector2> checkpoints;

	private Player player;

	private List<ParralaxMap> lstParralax;

	private int garbageCpt = 0;

	private GameHud hud;

	private Game game;

	/**
	 * Constructeur
	 */
	public GameMenu(){
	}


	public void goToTitleMenu() {
		game.goToTitleMenu();
	}
	public void goToMissionCompleteMenu() {
		game.goToEndgameMenu(hud);
	}

	public void goToCheckpoint() {
		Vector2 lastCheckpoint = checkpoints.get(0);
		for (Vector2 v : checkpoints) {
			if(v.y < player.getY()) {
				lastCheckpoint = v;
			}else {
				break;
			}
		}

		float cameraY = lastCheckpoint.y - player.getY();
		player.goToPosition(lastCheckpoint.x, lastCheckpoint.y);
		camera.translate(0,cameraY);
		camera.update();

		Ressources.MUSIC_01.setVolume(0.6f);
		Ressources.MUSIC_01.play();
		Ressources.MUSIC_01.setLooping(true);
	}
	/**
	 * Initialisation du menu de jeu
	 */
	public void init(Game game) {
		this.game = game;
		Shared.initBloodAnimation();

		Shared.SHAPE_RENDERER.setAutoShapeType(true);
		layer0_entities = new ArrayList<>();
		layer1_entities = new ArrayList<>();
		layer2_entities = new ArrayList<>();
		tmp_layer0_entities = new ArrayList<>();
		tmp_layer1_entities = new ArrayList<>();
		tmp_layer2_entities = new ArrayList<>();

		level1 = new GameMap();
		level1.init(this,1);

		layer1_entities.add(new GameMapItem(this, MapItem.GUN1, 512, 1952, 32, 32, 1));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 336, 2096, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 296, 1860, 32, 32, 0,true));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 410, 1920, 32, 32, 0,true));

		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 433, 3802, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 369, 3819, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 275, 5032, 32, 32, 0,true));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 245, 5032, 32, 32, 0));
		layer1_entities.add(new GameMapItem(this, MapItem.BOX1, 275, 5574, 32, 32, 1,true));
		layer1_entities.add(new GameMapItem(this, MapItem.BOX1, 233, 5574, 32, 32, 1));
		layer1_entities.add(new GameMapItem(this, MapItem.BOX1, 210, 5531, 32, 32, 1,true));
		layer1_entities.add(new GameMapItem(this, MapItem.BOX1, 210, 5491, 32, 32, 1,true));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 374, 8310, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 431, 8279, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 507, 8603, 32, 32, 0,true));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 258, 8591, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.BOX1, 180, 8634, 32, 32, 0,true));


		layer0_entities.add(new GameMapItem(this, MapItem.HEART, 355, 1606, 32, 32, 0));
		layer2_entities.add(new GameMapItem(this, MapItem.HEART, 553, 3143, 32, 32, 2));
		layer2_entities.add(new GameMapItem(this, MapItem.HEART, 581, 7790, 32, 32, 2));

		layer0_entities.add(new GameMapItem(this, MapItem.MEDKIT, 113, 2600, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.MEDKIT, 582, 4400, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.MEDKIT, 496, 4960, 32, 32, 0));

		layer0_entities.add(new GameMapItem(this, MapItem.LETTER, 200, 1394, 32, 32, 0,'B'));
		layer2_entities.add(new GameMapItem(this, MapItem.LETTER, 546, 6083, 32, 32, 2,'i'));
		layer0_entities.add(new GameMapItem(this, MapItem.LETTER, 240, 6384, 32, 32, 0,'g'));
		layer2_entities.add(new GameMapItem(this, MapItem.LETTER, 500, 7850, 32, 32, 2,'b'));
		layer0_entities.add(new GameMapItem(this, MapItem.LETTER, 500, 9231, 32, 32, 0,'o'));

		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,594, 1038, 32, 32, 0));
		layer1_entities.add(new GameMapItem(this, MapItem.DOLLARS,117, 2482, 32, 32, 1));
		layer2_entities.add(new GameMapItem(this, MapItem.DOLLARS,522, 2672, 32, 32, 2));
		layer2_entities.add(new GameMapItem(this, MapItem.DOLLARS,492, 2746, 32, 32, 2));
		layer2_entities.add(new GameMapItem(this, MapItem.DOLLARS,582, 3054, 32, 32, 2));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,78, 3579,  32, 32,0 ));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,499, 3873, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,558, 3935, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,544, 4022, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,138, 3816, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,68, 3885,  32, 32,0 ));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,63, 3984,  32, 32,0 ));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,111, 4307, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,532, 5308, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,330, 6551, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,562, 6791, 32, 32, 0));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,80, 6961,  32, 32,0 ));
		layer0_entities.add(new GameMapItem(this, MapItem.DOLLARS,322, 7556, 32, 32, 0));


		layer0_entities.add(new Zombi(this, 386, 2330, 0,0));
		layer0_entities.add(new Zombi(this, 418, 2589, 0,0));
		layer0_entities.add(new Zombi(this, 120, 2896, 0,0));
		layer0_entities.add(new Zombi(this, 412, 3167, 0,0));
		layer0_entities.add(new Zombi(this, 412, 3525, 0,0));
		layer0_entities.add(new Zombi(this, 123, 3491, 0,0));
		layer0_entities.add(new Zombi(this, 184, 3348, 0,0));

		layer0_entities.add(new Zombi(this, 341,3823,0 , 0));
		layer0_entities.add(new Zombi(this, 199,3823,0 , 1));
		layer0_entities.add(new Zombi(this, 467,3823,0 , 0));

		layer0_entities.add(new Zombi(this, 280,4096,0 , 0));
		layer0_entities.add(new Zombi(this, 350,4096,0 , 0));
		layer0_entities.add(new Zombi(this, 352,4313,0 , 1));
		layer0_entities.add(new Zombi(this, 558,4323,0 , 0));
		layer0_entities.add(new Zombi(this, 100,4292,0 , 0));
		layer0_entities.add(new Zombi(this, 417,4628,0 , 1));
		layer0_entities.add(new Zombi(this, 226,4616,0 , 1));
		layer0_entities.add(new Zombi(this, 277,4881,0 , 0));
		layer0_entities.add(new Zombi(this, 480,4495,0 , 1));
		layer0_entities.add(new Zombi(this, 342,4779,0 , 1));
		layer0_entities.add(new Zombi(this, 101,4750,0 , 0));

		layer0_entities.add(new Zombi(this, 478, 6628,0,0));
		layer0_entities.add(new Zombi(this, 400, 6692,0,1));
		layer0_entities.add(new Zombi(this, 311, 6727,0,1));
		layer0_entities.add(new Zombi(this, 282, 6814,0,1));
		layer0_entities.add(new Zombi(this, 388, 6906,0,0));
		layer0_entities.add(new Zombi(this, 439, 7094,0,0));
		layer0_entities.add(new Zombi(this, 314, 7148,0,1));
		layer0_entities.add(new Zombi(this, 173, 7148,0,1));

		layer0_entities.add(new Zombi(this,303, 5290,0,0));
		layer0_entities.add(new Zombi(this,452, 5354,0,0));
		layer0_entities.add(new Zombi(this,266, 8254,0,1));
		layer0_entities.add(new Zombi(this,406, 8324,0,0));
		layer0_entities.add(new Zombi(this,406, 8395,0,1));
		layer0_entities.add(new Zombi(this,318, 8421,0,1));
		layer0_entities.add(new Zombi(this,234, 8454,0,0));
		layer0_entities.add(new Zombi(this,234, 8526,0,1));
		layer0_entities.add(new Zombi(this,249, 8620,0,0));
		layer0_entities.add(new Zombi(this,293, 8654,0,1));
		layer0_entities.add(new Zombi(this,382, 8654,0,1));
		layer0_entities.add(new Zombi(this,472, 8654,0,0));
		layer0_entities.add(new Zombi(this,494, 8720,0,1));
		layer0_entities.add(new Zombi(this,494, 8795,0,0));
		layer0_entities.add(new Zombi(this,456, 8847,0,1));
		layer0_entities.add(new Zombi(this,487, 8926,0,0));
		layer0_entities.add(new Zombi(this,439, 8996,0,1));
		layer0_entities.add(new Zombi(this,331, 8973,0,0));
		layer0_entities.add(new Zombi(this,205, 8943,0,0));
		layer0_entities.add(new Zombi(this,126, 8859,0,1));
		layer0_entities.add(new Zombi(this,195, 8789,0,1));
		layer0_entities.add(new Zombi(this,201, 8702,0,1));
		layer0_entities.add(new Zombi(this,176, 9100,0,0));

		layer0_entities.add(new Zombi(this,353, 8465,0,2));
		layer0_entities.add(new Zombi(this,159, 8509,0,2));


		player = new Player(320,224);
		player.init(this);


		lstParralax = new ArrayList<>();
		lstParralax.add(new ParralaxMap(this,Ressources.TX_PARRALAX_LVL1,360,800,0,0,0f,0.3f));

		checkpoints = new ArrayList<>();
		checkpoints.add(new Vector2(320,224));
		checkpoints.add(new Vector2(334,1210));
		checkpoints.add(new Vector2(348,2245));
		checkpoints.add(new Vector2(334,5310));
		checkpoints.add(new Vector2(468,6582));
		checkpoints.add(new Vector2(228,7134));
		hud = new GameHud();
		hud.init(this);

		Ressources.MUSIC_01.play();
		Ressources.MUSIC_01.setLooping(true);
	}

	/**
	 * Mise a jour du jeu
	 * @param game le launcher
	 */
	public void update(Game game){


		Shared.GAME_BATCH.setProjectionMatrix(camera.combined);
		Shared.SHAPE_RENDERER.setProjectionMatrix(camera.combined);
		level1.update(this);

		if(!hud.isGameOver()) {
			// mise  jour du joueur
			player.update(this);

			for (Entity e : layer0_entities) {
				e.update(this);
			}
			for (Entity e : layer1_entities) {
				e.update(this);
			}
			for (Entity e : layer2_entities) {
				e.update(this);
			}

			// mise  jour des entites
			for (ParralaxMap p : lstParralax) {
				p.update(this);
			}
		}


		this.updateGarbage();

		layer0_entities.addAll(tmp_layer0_entities);
		layer1_entities.addAll(tmp_layer1_entities);
		layer2_entities.addAll(tmp_layer2_entities);

		tmp_layer0_entities.clear();
		tmp_layer1_entities.clear();
		tmp_layer2_entities.clear();

		hud.update(this);

		if(player.getY()>9300 && !hud.isMissionComplete()) {
			hud.missionComplete();
		}
	}

	/**
	 * Vide les objets dtruits des diffrentes listes
	 */
	private void updateGarbage() {
		if(garbageCpt >= 100) {
			garbageCpt = 0;
		}else {
			garbageCpt++;
		}


		if(garbageCpt == 33) {
			clearDestroyEntitiesFromList(layer0_entities);
		}else if(garbageCpt == 66) {
			clearDestroyEntitiesFromList(layer1_entities);
		}else if(garbageCpt == 99) {
			clearDestroyEntitiesFromList(layer2_entities);
		}
	}

	/**
	 * Nettoie les objets dtruits d'une liste
	 * @param lst la liste a nettoyer
	 */
	private void clearDestroyEntitiesFromList(List<Entity> lst) {
		Iterator<Entity> it = lst.iterator();
		Entity e;
		while(it.hasNext()) {
			e = it.next();
			if(e.isDestroy()) {
				it.remove();
			}
		}
	}

	/**
	 * Dessin du jeu
	 * @param game le launcher
	 */
	public void render(Game game){
		level1.render(this);

		// Render de la map en couche 0
		level1.renderLayer0();


		// Render des objets en couche 0
		Shared.GAME_BATCH.begin();
		for (Entity e : layer0_entities) {
			e.render(this);
		}
		if(player.getLevel() == 0) {
			// mise  jour du joueur
			player.render(this);
		}
		Shared.GAME_BATCH.end();

		// Render de la map en couche 1
		level1.renderLayer1();


		// Render des objets en couche 0
		Shared.GAME_BATCH.begin();
		for (Entity e : layer1_entities) {
			e.render(this);
		}
		if(player.getLevel() == 1) {
			// mise  jour du joueur
			player.render(this);
		}
		Shared.GAME_BATCH.end();		

		// Render de la map en couche 2
		level1.renderLayer2();

		// Render des objets en couche 0
		Shared.GAME_BATCH.begin();
		for (Entity e : layer2_entities) {
			e.render(this);
		}
		if(player.getLevel() >= 2) {
			// mise  jour du joueur
			player.render(this);
		}


		// Affichage des parralax
		for (ParralaxMap p : lstParralax) {
			p.render(this);
		}
		Shared.GAME_BATCH.end();

		this.renderDebug(game);

		Shared.MENU_BATCH.begin();
		hud.render(this);
		Shared.MENU_BATCH.end();
	}

	/**
	 * Dessin des informations de debug du jeu
	 * @param game le launcher
	 */
	private void renderDebug(Game game) {
		if(Shared.DEBUG) {
			Shared.SHAPE_RENDERER.begin();
			// mise  jour du joueur
			player.renderDebug(this);

			for (Entity e : layer0_entities) {
				e.renderDebug(this);
			}
			for (Entity e : layer1_entities) {
				e.renderDebug(this);
			}
			for (Entity e : layer2_entities) {
				e.renderDebug(this);
			}
			Shared.SHAPE_RENDERER.end();
		}
	}


	public void translateCamera(float dx, float dy) {
		camera.translate(dx, dy);
		// mise  jour des entites
		for (ParralaxMap p : lstParralax) {
			p.move(this,dx,dy);
		}
	}

	public GameMap getMap() {
		return level1;
	}

	public Player getPlayer() {
		return player;
	}

	public List<Entity> getLayerEntities(int layerIndex){
		if(layerIndex == 0) {
			return layer0_entities;
		}
		if(layerIndex == 1) {
			return layer1_entities;
		}
		if(layerIndex == 2) {
			return layer2_entities;
		}
		if(layerIndex == 3) {
			// TODO A REVOIR
			return layer2_entities;
		}
		return null;
	}

	public List<Entity> getTmpLayerEntities(int layerIndex){
		if(layerIndex == 0) {
			return tmp_layer0_entities;
		}
		if(layerIndex == 1) {
			return tmp_layer1_entities;
		}
		if(layerIndex == 2) {
			return tmp_layer2_entities;
		}
		if(layerIndex == 3) {
			// TODO A REVOIR
			return tmp_layer2_entities;
		}
		return null;
	}


	public GameHud getHud() {
		return hud;
	}
}
